@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -84,11 +85,39 @@
 #include "AFPUsers.h"
 #include "NetBootServer.h"
 
+/* 
+ * Function: timestamp_syslog
+ *
+ * Purpose:
+ *   Log a timestamped event message to the syslog.
+ */
+static void
+S_timestamp_syslog(char * msg)
+{
+    static struct timeval	tvp = {0,0};
+    struct timeval		tv;
+
+    gettimeofday(&tv, 0);
+    if (tvp.tv_sec) {
+	struct timeval result;
+      
+	timeval_subtract(tv, tvp, &result);
+	syslog(LOG_INFO, "%d.%06d (%d.%06d): %s", 
+	       tv.tv_sec, tv.tv_usec, result.tv_sec, result.tv_usec, msg);
+    }
+    else 
+	syslog(LOG_INFO, "%d.%06d (%d.%06d): %s", 
+	       tv.tv_sec, tv.tv_usec, 0, 0, msg);
+    tvp = tv;
+}
+
+
+
 static __inline__ void
 S_timestamp(char * msg)
 {
     if (verbose)
-	timestamp_syslog(msg);
+	S_timestamp_syslog(msg);
 }
 
 static boolean_t
