@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -37,8 +38,9 @@
 #include <stdio.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <ctype.h>
+#include <string.h>
 #include <sys/param.h>
-#include "util.h"
 #include "cfutil.h"
 
 #include <CoreFoundation/CFData.h>
@@ -161,29 +163,3 @@ my_CFPropertyListWriteFile(CFPropertyListRef plist, char * filename)
     return (ret);
 }
 
-Boolean
-DNSHostNameStringIsClean(CFStringRef str)
-{
-    char *		c_str = NULL;
-    Boolean		is_clean = FALSE;
-    CFIndex		len_str = 0;
-
-    len_str = CFStringGetLength(str) + 1;
-    c_str = CFAllocatorAllocate(NULL, len_str, 0);
-    if (c_str == NULL) {
-	goto failed;
-    }
-    if (CFStringGetCString(str, c_str, len_str, 
-			   kCFStringEncodingASCII) == FALSE) {
-	goto failed;
-    }
-    if (dns_hostname_is_clean(c_str) == FALSE) {
-	goto failed;
-    }
-    is_clean = TRUE;
- failed:
-    if (c_str != NULL) {
-	CFAllocatorDeallocate(NULL, c_str);
-    }
-    return (is_clean);
-}
